function objfun_tr(x0) result(logpost)

  real(wp), intent(in) :: x0(npara)
  real(wp) :: logpost

  real(wp) :: p0(npara)

  p0 = prtr(x0)

  logpost = objfunkf_missing(p0)

end function objfun_tr


subroutine minfunc(val, n, x, grad, need_gradient, f_data)

  integer :: n, need_gradient
  real(wp) :: val, x(n), grad(n), f_data
  val = -objfun_tr(x)
  print*,val
end subroutine

function prdiff(p0) result(logprdiff)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logprdiff

  if (inbounds(p0) .ne. .true.) then
     logprdiff = REALLY_NEG
     return
  end if
  
  logprdiff = priordens(p0,pmean2,pstdd2,pshape2,pmask2,pfix2) - &
       priordens(p0,pmean,pstdd,pshape,pmask,pfix)

end function prdiff

function pr(p0) result(logpr)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpr

  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     logpr = REALLY_NEG
     return
  end if

  logpr = priordens(p0,pmean,pstdd,pshape,pmask,pfix)

end function pr

function lik_missing(p0) result(loglh)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpri
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)

  real(wp),allocatable :: TTred(:,:), ZZred(:,:), RRred(:,:)
  integer :: ns1

  
  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     loglh = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)

  if (info .ne. 1) then
     loglh = REALLY_NEG
     return
  end if

 loglh = kalman_filter_missing(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)
!  call reduce_system(TT,RR,ZZ,TTred,RRred,ZZred,ny,ns,neps,ns1,.true.)
!  allocate(TTred(ns1,ns1),RRred(ns1,neps),ZZred(ny,ns1))
!  call reduce_system(TT,RR,ZZ,TTred,RRred,ZZred,ny,ns,neps,ns1,.false.)


 ! loglh = kalman_filter_missing(YY,TTred,RRred,QQ,DD,ZZred,HH,ny,nobs,neps,ns1,t0)

!  deallocate(TTred,RRred,ZZred)
end function lik_missing

subroutine write_model_para(basedir)
  
  character(len=*), intent(in) :: basedir

  character(len=144) :: ff3

  ff3 = trim(adjustl(basedir))//'/model_spec.txt'

  open(675,file=ff3,action='write')
  write(675,'(A,A)') 'mname        :', mname
  write(675,'(A,A)') 'priorfile    :', priorfile
  write(675,'(A,A)') 'datafile     :', datafile
  write(675,'(A,A)') 'transfile    :', transfile
  close(675)
end subroutine
function lik(p0) result(loglh)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpri
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)

  
  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     loglh = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)

  if (info .ne. 1) then
     loglh = REALLY_NEG
     return
  end if


!  loglh = kalman_filter(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)
  loglh = chand_recursion(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)

end function lik

function dlyap_test(p0) result(loglh)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpr
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny), Pt(ns,ns), Ft(ny,ny)
  real(wp) :: PtZZp(ns,ny), QQRRp(neps,ns), RRQQRRp(ns,ns)

  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     loglh = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)

  if (info .ne. 1) then
     loglh = REALLY_NEG
     return
  end if
  
  call dgemm('n','t', neps, ns, neps, 1.0_wp, QQ, neps, RR, ns, 0.0_wp, QQRRp, neps)
  call dgemm('n','n', ns, ns, neps, 1.0_wp, RR, ns, QQRRp, neps, 0.0_wp, RRQQRRp, ns)

  !call dlyap_symm(TT, RRQQRRp, Pt, ns, info)

  open(1,file='newsTT.txt',action='write')
  open(2,file='newsRQR.txt',action='write')
  do info = 1,ns
     write(1,'(98f)') TT(info,:)
     write(2,'(98f)') RRQQRRp(info,:)
  end do
  close(1)
  close(2)
  print*,'sum Pt = ', sum(Pt)
  call dgemm('n','t', ns, ny, ns, 1.0_wp, Pt, ns, ZZ, ny, 0.0_wp, PtZZp, ns)

  Ft = HH
  call dgemm('n','n', ny,  ny, ns, 1.0_wp, ZZ, ny, PtZZp, ns, 1.0_wp, Ft, ny)


  print*,'---------------------------------------------'
  do info = 1,ny
     !write(*,'(7f8.4)') F(info,:)
     print*, sqrt(Ft(info,info)), Ft(1,info)/sqrt(Ft(1,1)*Ft(info,info))
  end do
  print*,'---------------------------------------------'
  loglh = chand_recursion(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)
  print*, loglh
  !logpr = priordens(p0,pmean,pstdd,pshape,pmask,pfix)
  !print*, logpr, loglh, logpr + loglh
  print*,''

end function dlyap_test

function likT(p0, T1) result(loglh)

  real(wp), intent(inout) :: p0(npara)
  integer, intent(in) :: T1
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpri
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)


  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     loglh = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)

  if (info .ne. 1) then
     loglh = REALLY_NEG
     return
  end if

  loglh = chand_recursion(YY(:,1:T1),TT,RR,QQ,DD,ZZ,HH,ny,T1,neps,ns,t0)

end function likT

function objfun(p0) result(logpost)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpri
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)

  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     logpost = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)
  loglh = chand_recursion(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)

  logpri = priordens(p0,pmean,pstdd,pshape,pmask,pfix)
  logpost = loglh + logpri

end function objfun


function objfunkf(p0) result(logpost)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpri
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)

  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     logpost = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)
  loglh = kalman_filter(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)

  logpri = priordens(p0,pmean,pstdd,pshape,pmask,pfix)
  logpost = loglh + logpri

end function objfunkf


function objfunkf_missing(p0) result(logpost)

  real(wp), intent(inout) :: p0(npara)
  real(wp) :: logpost

  integer :: info
  real(wp) :: loglh, logpri
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)


  ! check whether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     logpost = REALLY_NEG
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)


  loglh = kalman_filter_missing(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)

  logpri = priordens(p0,pmean,pstdd,pshape,pmask,pfix)
  logpost = loglh + logpri

end function objfunkf_missing


subroutine lik_missing_with_states(p0,loglh,smooth_states,smooth_vars,smooth_shocks) 

  real(wp), intent(inout) :: p0(npara)
  real(wp), intent(out) :: loglh, smooth_states(nobs,ns), smooth_shocks(nobs,neps), smooth_vars(ns,ns,nobs)

  integer :: info
  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)


  ! check xowhether the parameters are inbounds 
  if (inbounds(p0) .ne. .true.) then
     loglh = REALLY_NEG
     smooth_states = 0.0_wp
     return
  end if

  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)

  call kalman_filter_missing_with_states(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0,loglh,smooth_states,smooth_vars,smooth_shocks)


end subroutine lik_missing_with_states



!!$function objfunblockkf(p0) result(logpost)
!!$
!!$  real(wp), intent(inout) :: p0(npara)
!!$  real(wp) :: logpost
!!$
!!$  integer :: info
!!$  real(wp) :: loglh, logpri
!!$  real(wp) :: TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny)
!!$
!!$
!!$  ! check whether the parameters are inbounds 
!!$  if (inbounds(p0) .ne. .true.) then
!!$     logpost = REALLY_NEG
!!$     return
!!$  end if
!!$
!!$  call sysmat(p0,TT,RR,QQ,DD,ZZ,HH,info)
!!$  loglh = 1.0_wp
!!$  loglh = sw_block_kf(YY,TT,RR,QQ,DD,ZZ,HH,ny,nobs,neps,ns,t0)
!!$  logpri = priordens(p0,pmean,pstdd,pshape,pmask,pfix)
!!$  logpost = loglh + logpri
!!$
!!$end function objfunblockkf


function prtr(p0) result(x0)
  ! translates original model parameters into unbounded space
  real(wp), intent(in) :: p0(npara)
  real(wp) :: x0(npara)

  integer :: i
  real(wp) :: a, b, c

  do i = 1, npara
     a = trspec(2,i)
     b = trspec(3,i)
     c = trspec(4,i)

     if (trspec(1,i) == 1.0_wp) then
        x0(i) = (a+b)/2.0_wp + 0.5_wp*(b-a)*c*p0(i)/sqrt(1.0_wp + c**2*p0(i)**2)
     elseif (trspec(1,i) == 2.0_wp) then
        x0(i) = a + exp(c*(p0(i) - b))
     else
        x0(i) = p0(i)
     endif

  end do

end function prtr

function iprtr(x0) result(p0)
  ! the inverse of prtr
  real(wp), intent(in) :: x0(npara)
  real(wp) :: p0(npara)

  integer :: i
  real(wp) :: a, b, c, cx

  do i = 1, npara
     a = trspec(2,i)
     b = trspec(3,i)
     c = trspec(4,i)

     if (trspec(1,i) == 1.0_wp) then
        cx = 2.0_wp*(x0(i) - (a+b)/2.0)/(b-a)
        p0(i) = 1.0_wp/c*cx/sqrt(1.0_wp-cx**2)
     elseif (trspec(1,i) == 2.0_wp) then
        p0(i) = b + 1.0/c*log(x0(i) - a)
     else
        p0(i) = x0(i) 
     endif

  end do

end function iprtr

function inbounds(p0)

  real(wp), intent(inout) :: p0(npara)
  logical :: inbounds

  real(wp) :: low, hi
  integer :: i
  
  complex(wp) :: rroot(2)
  real(wp) :: modroot(2)
  
  do i = 1, npara
     low = trspec(2,i)
     hi = trspec(3,i)

    if (pmask(i)==1) then
        p0(i) = pfix(i)
     end if

     if ((p0(i) < low .or. p0(i) > hi) .and. (pmask(i)==0)) then
        !print*,'parameter ', i, 'out of bounds', p0(i), low, hi
        inbounds = .false.
        return
     end if

 
  end do

  inbounds = .true.

  ! this is only for the RBC model
!!$  if (mname == 'rbc') then
!!$     hi = (p0(6)+p0(7))
!!$     low = p0(6)*p0(7)-p0(10)*p0(11)
!!$
!!$     rroot(1) = (hi - sqrt(hi**2 - 4.0_wp*low))/(2.0_wp)
!!$     rroot(2) = (hi + sqrt(hi**2 - 4.0_wp*low))/(2.0_wp)
!!$
!!$     modroot = sqrt(real(rroot)**2 + aimag(rroot)**2)
!!$
!!$     !print*,maxval(abs(modroot))
!!$     if (maxval(abs(modroot)) > 1.0_wp) then
!!$        inbounds = .false.
!!$     endif
!!$
!!$     if (any(isnan(modroot))) then
!!$        inbounds = .false.
!!$     end if
!!$  end if

end function inbounds


function any_missing_data(YY) result(ismiss)

  real(wp), intent(in) :: YY(ny,nobs)
  logical :: ismiss
  integer :: nmiss

  ismiss = any(isnan(YY))

end function

subroutine write_state_space(YY,TT,RR,QQ,DD,ZZ,HH)

  real(wp),intent(in) ::   TT(ns,ns), RR(ns,neps), QQ(neps,neps), DD(ny), ZZ(ny,ns), HH(ny,ny), YY(ny,nobs)

  integer :: i
  

  open(1,file='TT.txt',action='write')
  open(2,file='RR.txt',action='write')
  open(3,file='QQ.txt',action='write')
  open(4,file='DD.txt',action='write')
  open(5,file='HH.txt',action='write')
  open(6,file='ZZ.txt',action='write')
  open(7,file='YY.txt',action='write')


  do i = 1,ny
     write(6,'(1000f)') ZZ(i,:)
     write(4,'(1000f)') DD(i)
     write(5,'(1000f)') HH(i,:)
  end do

  do i = 1,neps
     write(3,'(1000f)') QQ(i,:)
  end do

  do i = 1,ns
     write(1,'(1000f)') TT(i,:)
     write(2,'(1000f)') RR(i,:)
  end do

  do i = 1,nobs
     write(7,'(1000f)') YY(:,i)
  end do

    close(1)
    close(2)
    close(3)
    close(4)
    close(5)
    close(6)
    close(7)

end subroutine write_state_space

