! file: compare_algorithms.f90
!
!   Compares
!      - Standard Kalman Filtering equations.
!      - The Strid-Walentin (2009) Block KF algorithm.
!      - The Chandrasekhar-type recursions algorithm (Morf et al. 1974).
!
!   By evaluating the likelihood of Smets-Wouters Model `nsim' times.
!
! Implemented by Ed Herbst <edward.p.herbst@frb.gov
!-------------------------------------------------------------------------------
program compare_algorithms
  use mkl95_precision, only: wp => dp

  !use news
  !use generic_state_space
  !use rbc
  use news

  implicit none

  include 'revision.inc'

  integer, parameter :: nsim = 1000

  ! The system and data matrices of the Smets-Wouters Model
  ! 
  ! We avoid solving the model here, which makes the comparison to the matlab
  !  programs difficult.   

  real(wp) :: lnp0, para(npara)


  ! counters and timing
  integer :: i, start_time, finish_time, cr, cm
  real(wp) :: rate


  print*, revision
  print*, 'model : ', mname
  ! wall timing initialization
  call system_clock(count_rate=cr)
  call system_clock(count_max=cm)
  rate = real(cr)


  open(1, file=priorfile, status='old', action='read')
  do i = 1, npara
     read(1, *) pshape(i), pmean(i), pstdd(i), pmask(i), pfix(i)
  end do
  close(1)

  open(1, file=datafile, status='old', action='read')
  do i = 1, nobs
     read(1, *) YY(:,i)
  end do
  close(1)

  open(1, file=transfile, status='old', action='read')
  do i = 1, npara
     read(1, *) trspec(:,i)

  end do
  close(1)

  para = pmsv()

  call system_clock(start_time)
  do i = 1, nsim
     lnp0 = objfunkf(para)
  end do
  call system_clock(finish_time)
  write(*,'(a,f7.3,a)') "Elapsed time is ", (finish_time-start_time)/rate, " seconds."
  write(*,'(a,f12.4)') "Kalman Filter:                  f(p0) = ", lnp0

  select case(mname)
  case('sw','sw-noma','rbc')

     call system_clock(start_time)
     do i = 1, nsim
        !lnp0 = objfunblockkf(para)
     end do
     call system_clock(finish_time)
     write(*,'(a,f7.3,a)') "Elapsed time is ", (finish_time-start_time)/rate, " seconds."
     write(*,'(a,f12.4)') "Block Kalman Filter:            f(p0) = ", lnp0

  end select

  call system_clock(start_time)
  do i = 1, nsim
     lnp0 = objfun(para)
  end do
  call system_clock(finish_time)
  write(*,'(a,f7.3,a)') "Elapsed time is ", (finish_time-start_time)/rate, " seconds."
  write(*,'(a,f12.4)') "Chandrasekhar-type recursions:  f(p0) = ", lnp0

end program compare_algorithms
