module news

  use mkl95_precision, only: wp => dp

  use prior
  use filter
  use gensys
!  use strid_walentin

  implicit none

  character(len=*), parameter :: mname = 'news-new' !'news-tilt-gam2'!-diffuse-notrunc'
  character(len=*), parameter :: priorfile = '/mq/home/m1eph00/code/fortran/models/news/prior.txt'!_gam_tilt2.txt'
  character(len=*), parameter :: priorfile2 = ''
  character(len=*), parameter :: datafile = '/mq/home/m1eph00/code/fortran/models/news/sgu_actual.txt'
  character(len=*), parameter :: transfile = '/mq/home/m1eph00/code/fortran/models/news/trspec.txt'

  character(len=*), parameter :: initfile = ''
  character(len=*), parameter :: initwt = ''

  character(len=*), parameter :: varfile = '/mq/home/m1eph00/code/fortran/models/news/news_tilt_gam_var.txt' !varfile = 'models/news/smc_final_cov.txt'

  integer, parameter :: neq = 98, neps = 21, neta = 14, npara = 45, ny = 7, nobs = 207, ns = 98
  integer, parameter :: t0 = 0

  real(wp), parameter :: REALLY_NEG = -10.0_wp**10

  !data stuff
  real(wp) :: YY(ny, nobs)

  ! prior stuff
  integer :: pshape(npara), pmask(npara)
  real(wp) :: pmean(npara), pstdd(npara), pfix(npara)

  integer :: pshape2(npara), pmask2(npara)
  real(wp) :: pmean2(npara), pstdd2(npara), pfix2(npara)


  ! prtr
  real(wp) :: trspec(4, npara)

contains

  include '/mq/home/m1eph00/code/fortran/base/helper_functions.f90'

  subroutine sysmat(para, TT, RR, QQ, DD, ZZ, HH, info)

    real(wp), intent(in) :: para(npara)

    integer, intent(out) :: info
    real(wp), intent(out) :: TT(neq, neq), RR(neq, neps), QQ(neps, neps)
    real(wp), intent(out) :: DD(ny), ZZ(ny, neq), HH(ny, ny)

    ! system matrices
    real(wp) :: GAM0(neq, neq), GAM1(neq, neq), C(neq), PSI(neq, neps), PPI(neq, neta), CC(neq), G0(neq, neq), G1(neq, neq)

    ! equation indices
    integer :: eq_muy, eq_muk, eq_xg, eq_mua, eq_mua8_1, eq_mua8_2
    integer :: eq_mua8_3, eq_mua8_4, eq_mua8_5, eq_mua8_6, eq_mua8_7, eq_mua8_8
    integer :: eq_mux, eq_mux8_1, eq_mux8_2, eq_mux8_3, eq_mux8_4, eq_mux8_5
    integer :: eq_mux8_6, eq_mux8_7, eq_mux8_8, eq_zi, eq_zi8_1, eq_zi8_2
    integer :: eq_zi8_3, eq_zi8_4, eq_zi8_5, eq_zi8_6, eq_zi8_7, eq_zi8_8, eq_z
    integer :: eq_z8_1, eq_z8_2, eq_z8_3, eq_z8_4, eq_z8_5, eq_z8_6, eq_z8_7
    integer :: eq_z8_8, eq_mu, eq_mu8_1, eq_mu8_2, eq_mu8_3, eq_mu8_4, eq_mu8_5
    integer :: eq_mu8_6, eq_mu8_7, eq_mu8_8, eq_g, eq_g8_1, eq_g8_2, eq_g8_3
    integer :: eq_g8_4, eq_g8_5, eq_g8_6, eq_g8_7, eq_g8_8, eq_zet, eq_zet8_1
    integer :: eq_zet8_2, eq_zet8_3, eq_zet8_4, eq_zet8_5, eq_zet8_6, eq_zet8_7
    integer :: eq_zet8_8, eq_glag, eq_xglag, eq_zlag, eq_inv, eq_res, eq_v, eq_s
    integer :: eq_c, eq_h, eq_p, eq_y, eq_cap, eq_eul, eq_pinv, eq_ylag, eq_clag
    integer :: eq_ilag, eq_hlag, eq_Elam, eq_Emuy, eq_Emuk, eq_Ei, eq_Eu, eq_Ez 
    integer :: eq_Eh, eq_Emua, eq_Eq, eq_Ep, eq_Es, eq_Ezet, eq_Ec, eq_Ev 

    !variable indices
    integer :: v_muy, v_muk, v_xg, v_mua, v_mua8_1, v_mua8_2, v_mua8_3, v_mua8_4
    integer :: v_mua8_5, v_mua8_6, v_mua8_7, v_mua8_8, v_mux, v_mux8_1, v_mux8_2
    integer :: v_mux8_3, v_mux8_4, v_mux8_5, v_mux8_6, v_mux8_7, v_mux8_8, v_zi
    integer :: v_zi8_1, v_zi8_2, v_zi8_3, v_zi8_4, v_zi8_5, v_zi8_6, v_zi8_7
    integer :: v_zi8_8, v_z, v_z8_1, v_z8_2, v_z8_3, v_z8_4, v_z8_5, v_z8_6
    integer :: v_z8_7, v_z8_8, v_mu, v_mu8_1, v_mu8_2, v_mu8_3, v_mu8_4, v_mu8_5
    integer :: v_mu8_6, v_mu8_7, v_mu8_8, v_g, v_g8_1, v_g8_2, v_g8_3, v_g8_4
    integer :: v_g8_5, v_g8_6, v_g8_7, v_g8_8, v_zet, v_zet8_1, v_zet8_2
    integer :: v_zet8_3, v_zet8_4, v_zet8_5, v_zet8_6, v_zet8_7, v_zet8_8
    integer :: v_glag, v_xglag, v_zlag, v_k1, v_u, v_i, v_c, v_y, v_s, v_h, v_p
    integer :: v_q, v_v, v_lam, v_ylag, v_clag, v_ilag, v_hlag, v_Elam, v_Emuy
    integer :: v_Emuk, v_Ei, v_Eu, v_Ez, v_Eh, v_Emua, v_Eq, v_Ep, v_Es, v_Ezet
    integer :: v_Ec, v_Ev

    ! shock indices
    integer :: e_mua0, e_mua4, e_mua8, e_mux0, e_mux4, e_mux8, e_zi0, e_zi4
    integer :: e_zi8, e_z0, e_z4, e_z8, e_mu0, e_mu4, e_mu8, e_g0, e_g4, e_g8
    integer :: e_zet0, e_zet4, e_zet8

    !! expectation errors
    integer :: sh_Elam, sh_Emuy, sh_Emuk, sh_Ei, sh_Eu, sh_Ez, sh_Eh, sh_Emua
    integer :: sh_Eq, sh_Ep, sh_Es, sh_Ezet, sh_Ec, sh_Ev

    !! observable equations
    integer :: eq_ygr, eq_cgr, eq_igr, eq_hgr, eq_ggr, eq_agr, eq_tfp

    !! parameter names
    real(wp):: thet, gam, kap, del2_del1, b, rhoxg, rhoz, rhoa, rhog, rhox, rhom, rhozet, rhozi	
    real(wp):: alpk, alph, del0, bet, hss, muss, muass, muxss, sig, g_y, xgss
    real(wp):: mukss,  muyss, y_k, i_k, i_y, c_y, y_c, betbar, badj, ppsi, del1, del2, xi, eta

    !! gensys stuff
    real(wp) :: fmat, fwt, ywt, gev, loose, DIV
    integer :: eu(2), i

    ! parameter vector
    thet      = para(1) + 1.0_wp;
    gam	      = para(2);
    kap	      = para(3);
    del2_del1 = para(4);
    b	      = para(5)*0.99_wp;
    rhoxg     = para(6)*0.99_wp;
    rhoz      = para(7)*0.99_wp;
    rhoa      = para(8)*0.99_wp;
    rhog      = para(9)*0.99_wp;
    rhox      = para(10); 
    rhom      = para(11)*0.99_wp;
    rhozet    = para(12)*0.99_wp;
    rhozi     = para(13)*0.99_wp;

    !! set calibrated parameters
    alpk  = para(14) 
    alph  = para(15) 
    del0  = para(16) 
    bet	  = para(17) 
    hss	  = para(18) 
    muss  = para(19) 
    muass = para(20) 
    muxss = para(21) 
    sig	  = para(22) 
    g_y	  = para(23) 

    !! find steady states, et cetera 
    mukss = muass**(1.0_wp/(alpk-1.0_wp))*muxss;! steady state growth for captial
    muyss = muass**(alpk/(alpk-1.0_wp))*muxss;	! steady state output growth rate
    xgss  = ((1.0_wp/muyss)**(1.0_wp/(1.0_wp - rhoxg)))

    g_y   = g_y/xgss;
    y_k   = (1.0_wp/(bet*muass*muyss**(-sig)) - (1.0_wp-del0))/(alpk*mukss);
    i_k   = 1.0_wp - (1.0_wp-del0)/mukss;
    i_y   = i_k/y_k;
    c_y   = 1.0_wp - g_y*xgss - i_y;
    y_c   = 1.0_wp/c_y;
    betbar= bet*b*muyss**(-sig);
    badj  = b/muyss;
    ppsi   = ((1.0_wp-betbar)*alph/(muss)*y_c)/(hss**thet*(1.0_wp/muyss)**((1.0_wp-gam)/gam)*(thet*(1.0_wp-badj)+gam/(1.0_wp-bet*(1.0_wp-gam)*muyss**(1.0_wp-sig))*alph/(muss)*y_c*(1.0_wp-betbar)));
    del1  = alpk*y_k*mukss;
    del2  = del2_del1*del1;
    xi    = 1.0_wp-bet*(1-gam)*muyss**(1.0_wp-sig);
    eta   = ppsi*hss**thet*gam*(1.0_wp/muyss)**((1.0_wp-gam)/gam)/xi;

    !! equations  
    eq_muy    = 1;
    eq_muk    = 2;
    eq_xg     = 3;
    eq_mua    = 4;
    eq_mua8_1 = 5;
    eq_mua8_2 = 6;
    eq_mua8_3 = 7;
    eq_mua8_4 = 8;
    eq_mua8_5 = 9;
    eq_mua8_6 = 10;
    eq_mua8_7 = 11;
    eq_mua8_8 = 12;
    eq_mux    = 13;
    eq_mux8_1 = 14;
    eq_mux8_2 = 15;
    eq_mux8_3 = 16;
    eq_mux8_4 = 17;
    eq_mux8_5 = 18;
    eq_mux8_6 = 19;
    eq_mux8_7 = 20;
    eq_mux8_8 = 21;
    eq_zi     = 22;
    eq_zi8_1  = 23;
    eq_zi8_2  = 24;
    eq_zi8_3  = 25;
    eq_zi8_4  = 26;
    eq_zi8_5  = 27;
    eq_zi8_6  = 28;
    eq_zi8_7  = 29;
    eq_zi8_8  = 30;
    eq_z      = 31;
    eq_z8_1   = 32;
    eq_z8_2   = 33;
    eq_z8_3   = 34;
    eq_z8_4   = 35;
    eq_z8_5   = 36;
    eq_z8_6   = 37;
    eq_z8_7   = 38;
    eq_z8_8   = 39;
    eq_mu     = 40;
    eq_mu8_1  = 41;
    eq_mu8_2  = 42;
    eq_mu8_3  = 43;
    eq_mu8_4  = 44;
    eq_mu8_5  = 45;
    eq_mu8_6  = 46;
    eq_mu8_7  = 47;
    eq_mu8_8  = 48;
    eq_g      = 49;
    eq_g8_1   = 50;
    eq_g8_2   = 51;
    eq_g8_3   = 52;
    eq_g8_4   = 53;
    eq_g8_5   = 54;
    eq_g8_6   = 55;
    eq_g8_7   = 56;
    eq_g8_8   = 57;
    eq_zet    = 58;
    eq_zet8_1 = 59;
    eq_zet8_2 = 60;
    eq_zet8_3 = 61;
    eq_zet8_4 = 62;
    eq_zet8_5 = 63;
    eq_zet8_6 = 64;
    eq_zet8_7 = 65;
    eq_zet8_8 = 66;
    eq_glag   = 80;
    eq_xglag  = 78;
    eq_zlag   = 79;
    eq_inv    = 67;
    eq_res    = 68;
    eq_v      = 69;
    eq_s      = 70;
    eq_c      = 71;
    eq_h      = 72;
    eq_p      = 73;
    eq_y      = 74;
    eq_cap    = 75;
    eq_eul    = 76;
    eq_pinv   = 77;
    eq_ylag   = 81;
    eq_clag   = 82;
    eq_ilag   = 83;
    eq_hlag   = 84;
    eq_Elam   = 85;
    eq_Emuy   = 86;
    eq_Emuk   = 87;
    eq_Ei     = 88;
    eq_Eu     = 89;
    eq_Ez     = 90;
    eq_Eh     = 91;
    eq_Emua   = 92;
    eq_Eq     = 93;
    eq_Ep     = 94;
    eq_Es     = 95;
    eq_Ezet   = 96;
    eq_Ec     = 97;
    eq_Ev     = 98;

    !! variables   
    v_muy    = 1;
    v_muk    = 2;
    v_xg     = 3;
    v_mua    = 4;
    v_mua8_1 = 5;
    v_mua8_2 = 6;
    v_mua8_3 = 7;
    v_mua8_4 = 8;
    v_mua8_5 = 9;
    v_mua8_6 = 10;
    v_mua8_7 = 11;
    v_mua8_8 = 12;
    v_mux    = 13;
    v_mux8_1 = 14;
    v_mux8_2 = 15;
    v_mux8_3 = 16;
    v_mux8_4 = 17;
    v_mux8_5 = 18;
    v_mux8_6 = 19;
    v_mux8_7 = 20;
    v_mux8_8 = 21;
    v_zi     = 22;
    v_zi8_1  = 23;
    v_zi8_2  = 24;
    v_zi8_3  = 25;
    v_zi8_4  = 26;
    v_zi8_5  = 27;
    v_zi8_6  = 28;
    v_zi8_7  = 29;
    v_zi8_8  = 30;
    v_z	     = 31;
    v_z8_1   = 32;
    v_z8_2   = 33;
    v_z8_3   = 34;
    v_z8_4   = 35;
    v_z8_5   = 36;
    v_z8_6   = 37;
    v_z8_7   = 38;
    v_z8_8   = 39;
    v_mu     = 40;
    v_mu8_1  = 41;
    v_mu8_2  = 42;
    v_mu8_3  = 43;
    v_mu8_4  = 44;
    v_mu8_5  = 45;
    v_mu8_6  = 46;
    v_mu8_7  = 47;
    v_mu8_8  = 48;
    v_g	     = 49;
    v_g8_1   = 50;
    v_g8_2   = 51;
    v_g8_3   = 52;
    v_g8_4   = 53;
    v_g8_5   = 54;
    v_g8_6   = 55;
    v_g8_7   = 56;
    v_g8_8   = 57;
    v_zet    = 58;
    v_zet8_1 = 59;
    v_zet8_2 = 60;
    v_zet8_3 = 61;
    v_zet8_4 = 62;
    v_zet8_5 = 63;
    v_zet8_6 = 64;
    v_zet8_7 = 65;
    v_zet8_8 = 66;
    v_glag   = 67;
    v_xglag  = 68;
    v_zlag   = 69;
    v_k1     = 70;
    v_u	     = 71;
    v_i	     = 72;
    v_c	     = 73;
    v_y	     = 74;
    v_s	     = 75;
    v_h	     = 76;
    v_p	     = 77;
    v_q      = 78;
    v_v      = 79;
    v_lam    = 80;
    v_ylag   = 81;
    v_clag   = 82;
    v_ilag   = 83;
    v_hlag   = 84;
    v_Elam   = 85;
    v_Emuy   = 86;
    v_Emuk   = 87;
    v_Ei     = 88;
    v_Eu     = 89;
    v_Ez     = 90;
    v_Eh     = 91;
    v_Emua   = 92;
    v_Eq     = 93;
    v_Ep     = 94;
    v_Es     = 95;
    v_Ezet   = 96;
    v_Ec     = 97;
    v_Ev     = 98;

    !! shocks
    e_mua0 = 1;
    e_mua4 = 2;
    e_mua8 = 3;
    e_mux0 = 4;
    e_mux4 = 5;
    e_mux8 = 6;
    e_zi0  = 7;
    e_zi4  = 8;
    e_zi8  = 9;
    e_z0   = 10;
    e_z4   = 11;
    e_z8   = 12;
    e_mu0  = 13;
    e_mu4  = 14;
    e_mu8  = 15;
    e_g0   = 16;
    e_g4   = 17;
    e_g8   = 18;
    e_zet0 = 19;
    e_zet4 = 20;
    e_zet8 = 21;

    !! expectation errors
    sh_Elam = 1;
    sh_Emuy = 2;
    sh_Emuk = 3;
    sh_Ei   = 4;
    sh_Eu   = 5;
    sh_Ez   = 6;
    sh_Eh   = 7;
    sh_Emua = 8;
    sh_Eq   = 9;
    sh_Ep   = 10;
    sh_Es   = 11;
    sh_Ezet = 12;
    sh_Ec   = 13;
    sh_Ev   = 14;


    !! definitions  
    GAM0 = 0.0_wp
    GAM1 = 0.0_wp
    PPI  = 0.0_wp
    PSI  = 0.0_wp
    C    = 0.0_wp

    TT = 0.0_wp
    RR = 0.0_wp
    QQ = 0.0_wp

    DD = 0.0_wp
    ZZ = 0.0_wp
    HH = 0.0_wp

    !!      1. Capital Accumulation                          !!

    GAM0(eq_inv, v_k1) = -1.0_wp;
    GAM0(eq_inv, v_u)  = -del1/mukss;
    GAM0(eq_inv, v_muk)= -(1.0_wp-del0)/mukss;
    GAM1(eq_inv, v_k1) = -(1.0_wp-del0)/mukss;
    GAM0(eq_inv, v_i)  =  i_k;
    GAM0(eq_inv, v_zi) =  i_k;


    !!      2. Resource Constraint                           !!

    GAM0(eq_res, v_c)   = c_y;
    GAM0(eq_res, v_i)   = i_y;
    GAM0(eq_res, v_g)   = g_y*xgss;
    GAM0(eq_res, v_xg)  = g_y*xgss;
    GAM0(eq_res, v_y)   = -1.0_wp;


    !!      3. Consumption Bundle                           !!

    GAM0(eq_v, v_v)   = -(1.0_wp-badj)*(1.0_wp-ppsi*hss**(thet)*(1/muyss)**((1.0_wp-gam)/gam));
    GAM0(eq_v, v_c)   =  1.0_wp;
    GAM0(eq_v, v_muy) =  badj;
    GAM1(eq_v, v_c)   =  badj;
    GAM0(eq_v, v_h)   = -ppsi*hss**thet*(1.0_wp-badj)*(1.0_wp/muyss)**((1.0_wp-gam)/gam)*thet;
    GAM0(eq_v, v_s)   = -ppsi*hss**thet*(1.0_wp-badj)*(1.0_wp/muyss)**((1.0_wp-gam)/gam);


    !!      4. Consumption Decision                          !!

    GAM0(eq_c, v_lam)  = -(1.0_wp-betbar)*(1.0_wp-eta);
    GAM0(eq_c, v_zet)  =  1.0_wp;
    GAM0(eq_c, v_v)    = -sig;
    GAM0(eq_c, v_c)    =  eta*(1.0_wp+betbar*badj)/(1.0_wp-badj);
    GAM1(eq_c, v_c)    =  eta*badj/(1.0_wp-badj);
    GAM0(eq_c, v_muy)  =  eta*badj/(1.0_wp-badj);
    GAM0(eq_c, v_p)    = -eta;
    GAM0(eq_c, v_s)    = -eta;    
    GAM0(eq_c, v_Ezet) = -betbar;
    GAM0(eq_c, v_Ev)   =  betbar*sig;
    GAM0(eq_c, v_Ep)   =  betbar*eta;
    GAM0(eq_c, v_Es)   =  betbar*eta;
    GAM0(eq_c, v_Ec)   = -betbar*eta/(1.0_wp-badj);
    GAM0(eq_c, v_Emuy) =  betbar*((1.0_wp-eta)*sig-eta*badj/(1.0_wp-badj));


    !!      5. Jaimovich-Rebelo Thing                        !!

    GAM0(eq_s,v_s)    = -1.0_wp;
    GAM0(eq_s,v_c)    =  gam/(1.0_wp-badj);
    GAM1(eq_s,v_c)    =  gam*badj/(1.0_wp-badj);
    GAM0(eq_s,v_muy)  =  gam*badj/(1.0_wp-badj)-(1.0_wp-gam);
    GAM1(eq_s,v_s)    = -(1.0_wp-gam);

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      6. Hours Decision                                !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM0(eq_h,v_zet)  =  1.0_wp;
    GAM0(eq_h,v_v)    = -sig;
    GAM0(eq_h,v_h)    =  thet;
    GAM0(eq_h,v_s)    =  1.0_wp;
    GAM0(eq_h,v_lam)  = -1.0_wp;
    GAM0(eq_h,v_y)    = -1.0_wp;
    GAM0(eq_h,v_mu)   =  1.0_wp; !The shock is on the gross mark-up

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      7. Shadow Price                                  !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM0(eq_p,v_p)    = -1.0_wp;
    GAM0(eq_p,v_zet)  =  xi;
    GAM0(eq_p,v_v)    = -xi*sig;
    GAM0(eq_p,v_h)    =  xi*thet;
    GAM0(eq_p,v_Emuy) =  (1.0_wp-xi)*(1.0_wp-sig);
    GAM0(eq_p,v_Ep)   =  1.0_wp-xi;
    GAM0(eq_p,v_Es)   =  1.0_wp-xi;
    GAM0(eq_p,v_s)    = -(1.0_wp-xi);

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      8. Output                                        !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM1(eq_y, v_k1) = -alpk;
    GAM0(eq_y, v_y)  = -1.0_wp;
    GAM0(eq_y, v_z)  =  1.0_wp;
    GAM0(eq_y, v_u)  =  alpk;
    GAM0(eq_y, v_muk)= -alpk;
    GAM0(eq_y, v_h)  =  alph;

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      9. Capacity Utilization                          !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM0(eq_cap, v_q)   = -1.0_wp;
    GAM0(eq_cap, v_z)   =  1.0_wp;
    GAM0(eq_cap, v_u)   =  alpk-1.0_wp-del2/del1;
    GAM1(eq_cap, v_k1)  = -(alpk-1.0_wp);
    GAM0(eq_cap, v_muk) = -(alpk-1.0_wp);
    GAM0(eq_cap, v_h)   =  alph;

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      10. Euler Equation                               !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM0(eq_eul, v_lam)  = -1.0_wp;
    GAM0(eq_eul, v_q)    = -1.0_wp;
    GAM0(eq_eul, v_Elam) =  1.0_wp;
    GAM0(eq_eul, v_Emua) =  1.0_wp;
    GAM0(eq_eul, v_Emuy) = -sig;
    GAM0(eq_eul, v_Ez)   =  bet*muass*muyss**(-sig)*del1;
    GAM0(eq_eul, v_Eu)   =  bet*muass*muyss**(-sig)*del1*(alpk-1.0_wp);
    GAM0(eq_eul, v_k1)   =  bet*muass*muyss**(-sig)*del1*(alpk-1.0_wp);
    GAM0(eq_eul, v_Emuk) = -bet*muass*muyss**(-sig)*del1*(alpk-1.0_wp);
    GAM0(eq_eul, v_Eh)   =  bet*muass*muyss**(-sig)*del1*alph;
    GAM0(eq_eul, v_Eq)   =  bet*muass*muyss**(-sig)*(1.0_wp-del0);


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      11. Value of Firm                                !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM0(eq_pinv, v_q)    =  1.0_wp;
    GAM0(eq_pinv, v_zi)   =  1.0_wp;
    GAM0(eq_pinv, v_i)    = -kap*(bet*muass*muyss**(-sig)*mukss**3 + mukss**2);
    GAM1(eq_pinv, v_i)    = -kap*mukss**2;
    GAM0(eq_pinv, v_muk)  = -kap*mukss**2;
    GAM0(eq_pinv, v_Ei)   =  kap*bet*muass*muyss**(-sig)*mukss**3;
    GAM0(eq_pinv, v_Emuk) =  kap*bet*muass*muyss**(-sig)*mukss**3;

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      12-14. Stochastic Trends                         !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    GAM0(eq_muy, v_muy) = -1.0_wp;
    GAM0(eq_muy, v_mua) =  alpk/(alpk-1.0_wp);
    GAM0(eq_muy, v_mux) =  1.0_wp;

    GAM0(eq_muk, v_muk) = -1.0_wp;
    GAM0(eq_muk, v_mua) =  1.0_wp/(alpk-1.0_wp);
    GAM0(eq_muk, v_mux) =  1.0_wp;

    GAM0(eq_xg,  v_xg ) =  1.0_wp;
    GAM1(eq_xg,  v_xg ) =  rhoxg;
    GAM0(eq_xg,  v_muy) =  1.0_wp;

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !!      15+. Shock Processes                             !!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    !! MUA !!
    GAM0(eq_mua,v_mua)    = 1.0_wp;
    GAM1(eq_mua,v_mua)    = rhoa;
    GAM1(eq_mua,v_mua8_1) = 1.0_wp;
    PSI(eq_mua,e_mua0) = 1.0_wp;

    GAM0(eq_mua8_1,v_mua8_1) = 1.0_wp
    GAM1(eq_mua8_1,v_mua8_2) = 1.0_wp

    GAM0(eq_mua8_2,v_mua8_2) = 1.0_wp
    GAM1(eq_mua8_2,v_mua8_3) = 1.0_wp

    GAM0(eq_mua8_3,v_mua8_3) = 1.0_wp
    GAM1(eq_mua8_3,v_mua8_4) = 1.0_wp

    GAM0(eq_mua8_4,v_mua8_4) = 1.0_wp
    GAM1(eq_mua8_4,v_mua8_5) = 1.0_wp
    PSI(eq_mua8_4,e_mua4)    = 1.0_wp

    GAM0(eq_mua8_5,v_mua8_5) = 1.0_wp
    GAM1(eq_mua8_5,v_mua8_6) = 1.0_wp

    GAM0(eq_mua8_6,v_mua8_6) = 1.0_wp
    GAM1(eq_mua8_6,v_mua8_7) = 1.0_wp

    GAM0(eq_mua8_7,v_mua8_7) = 1.0_wp
    GAM1(eq_mua8_7,v_mua8_8) = 1.0_wp

    GAM0(eq_mua8_8,v_mua8_8) = 1.0_wp
    PSI(eq_mua8_8,e_mua8)    = 1.0_wp

    !! MUX !!
    GAM0(eq_mux,v_mux)    = 1.0_wp
    GAM1(eq_mux,v_mux)    = rhox-0.5_wp;
    GAM1(eq_mux,v_mux8_1) = 1.0_wp
    PSI(eq_mux,e_mux0) = 1.0_wp

    GAM0(eq_mux8_1,v_mux8_1) = 1.0_wp	
    GAM1(eq_mux8_1,v_mux8_2) = 1.0_wp	

    GAM0(eq_mux8_2,v_mux8_2) = 1.0_wp	
    GAM1(eq_mux8_2,v_mux8_3) = 1.0_wp	

    GAM0(eq_mux8_3,v_mux8_3) = 1.0_wp	
    GAM1(eq_mux8_3,v_mux8_4) = 1.0_wp	

    GAM0(eq_mux8_4,v_mux8_4) = 1.0_wp	
    GAM1(eq_mux8_4,v_mux8_5) = 1.0_wp
    PSI(eq_mux8_4,e_mux4)     = 1.0_wp

    GAM0(eq_mux8_5,v_mux8_5) = 1.0_wp
    GAM1(eq_mux8_5,v_mux8_6) = 1.0_wp

    GAM0(eq_mux8_6,v_mux8_6) = 1.0_wp
    GAM1(eq_mux8_6,v_mux8_7) = 1.0_wp

    GAM0(eq_mux8_7,v_mux8_7) = 1.0_wp
    GAM1(eq_mux8_7,v_mux8_8) = 1.0_wp

    GAM0(eq_mux8_8,v_mux8_8) = 1.0_wp
    PSI(eq_mux8_8,e_mux8)   = 1.0_wp


    !! zi !!
    GAM0(eq_zi,v_zi)    = 1.0_wp
    GAM1(eq_zi,v_zi)    = rhozi;
    GAM1(eq_zi,v_zi8_1) = 1.0_wp
    PSI(eq_zi,e_zi0) = 1.0_wp

    GAM0(eq_zi8_1,v_zi8_1) = 1.0_wp
    GAM1(eq_zi8_1,v_zi8_2) = 1.0_wp

    GAM0(eq_zi8_2,v_zi8_2) = 1.0_wp
    GAM1(eq_zi8_2,v_zi8_3) = 1.0_wp

    GAM0(eq_zi8_3,v_zi8_3) = 1.0_wp
    GAM1(eq_zi8_3,v_zi8_4) = 1.0_wp

    GAM0(eq_zi8_4,v_zi8_4) = 1.0_wp
    GAM1(eq_zi8_4,v_zi8_5) = 1.0_wp
    PSI(eq_zi8_4,e_zi4)     = 1.0_wp

    GAM0(eq_zi8_5,v_zi8_5) = 1.0_wp
    GAM1(eq_zi8_5,v_zi8_6) = 1.0_wp

    GAM0(eq_zi8_6,v_zi8_6) = 1.0_wp
    GAM1(eq_zi8_6,v_zi8_7) = 1.0_wp

    GAM0(eq_zi8_7,v_zi8_7) = 1.0_wp
    GAM1(eq_zi8_7,v_zi8_8) = 1.0_wp

    GAM0(eq_zi8_8,v_zi8_8) = 1.0_wp
    PSI(eq_zi8_8,e_zi8)   = 1.0_wp

    !! z !! 
    GAM0(eq_z,v_z)    = 1.0_wp
    GAM1(eq_z,v_z)    = rhoz;
    GAM1(eq_z,v_z8_1) = 1.0_wp
    PSI(eq_z,e_z0) = 1.0_wp

    GAM0(eq_z8_1,v_z8_1) = 1.0_wp
    GAM1(eq_z8_1,v_z8_2) = 1.0_wp

    GAM0(eq_z8_2,v_z8_2) = 1.0_wp
    GAM1(eq_z8_2,v_z8_3) = 1.0_wp

    GAM0(eq_z8_3,v_z8_3) = 1.0_wp
    GAM1(eq_z8_3,v_z8_4) = 1.0_wp

    GAM0(eq_z8_4,v_z8_4) = 1.0_wp
    GAM1(eq_z8_4,v_z8_5) = 1.0_wp
    PSI(eq_z8_4,e_z4)     = 1.0_wp

    GAM0(eq_z8_5,v_z8_5) = 1.0_wp
    GAM1(eq_z8_5,v_z8_6) = 1.0_wp

    GAM0(eq_z8_6,v_z8_6) = 1.0_wp
    GAM1(eq_z8_6,v_z8_7) = 1.0_wp

    GAM0(eq_z8_7,v_z8_7) = 1.0_wp
    GAM1(eq_z8_7,v_z8_8) = 1.0_wp

    GAM0(eq_z8_8,v_z8_8) = 1.0_wp
    PSI(eq_z8_8,e_z8)   = 1.0_wp

    !! mu !!
    GAM0(eq_mu,v_mu)    = 1.0_wp
    GAM1(eq_mu,v_mu)    = rhom;
    GAM1(eq_mu,v_mu8_1) = 1.0_wp
    PSI(eq_mu,e_mu0) = 1.0_wp

    GAM0(eq_mu8_1,v_mu8_1) = 1.0_wp
    GAM1(eq_mu8_1,v_mu8_2) = 1.0_wp

    GAM0(eq_mu8_2,v_mu8_2) = 1.0_wp
    GAM1(eq_mu8_2,v_mu8_3) = 1.0_wp

    GAM0(eq_mu8_3,v_mu8_3) = 1.0_wp
    GAM1(eq_mu8_3,v_mu8_4) = 1.0_wp

    GAM0(eq_mu8_4,v_mu8_4) = 1.0_wp
    GAM1(eq_mu8_4,v_mu8_5) = 1.0_wp
    PSI(eq_mu8_4,e_mu4)     = 1.0_wp

    GAM0(eq_mu8_5,v_mu8_5) = 1.0_wp
    GAM1(eq_mu8_5,v_mu8_6) = 1.0_wp

    GAM0(eq_mu8_6,v_mu8_6) = 1.0_wp
    GAM1(eq_mu8_6,v_mu8_7) = 1.0_wp

    GAM0(eq_mu8_7,v_mu8_7) = 1.0_wp
    GAM1(eq_mu8_7,v_mu8_8) = 1.0_wp

    GAM0(eq_mu8_8,v_mu8_8) = 1.0_wp
    PSI(eq_mu8_8,e_mu8)   = 1.0_wp

    !! g !!
    GAM0(eq_g,v_g)    = 1.0_wp
    GAM1(eq_g,v_g)    = rhog;
    GAM1(eq_g,v_g8_1) = 1.0_wp
    PSI(eq_g,e_g0)    = 1.0_wp

    GAM0(eq_g8_1,v_g8_1) = 1.0_wp
    GAM1(eq_g8_1,v_g8_2) = 1.0_wp

    GAM0(eq_g8_2,v_g8_2) = 1.0_wp
    GAM1(eq_g8_2,v_g8_3) = 1.0_wp

    GAM0(eq_g8_3,v_g8_3) = 1.0_wp
    GAM1(eq_g8_3,v_g8_4) = 1.0_wp

    GAM0(eq_g8_4,v_g8_4) = 1.0_wp
    GAM1(eq_g8_4,v_g8_5) = 1.0_wp
    PSI(eq_g8_4,e_g4)     = 1.0_wp

    GAM0(eq_g8_5,v_g8_5) = 1.0_wp
    GAM1(eq_g8_5,v_g8_6) = 1.0_wp

    GAM0(eq_g8_6,v_g8_6) = 1.0_wp
    GAM1(eq_g8_6,v_g8_7) = 1.0_wp

    GAM0(eq_g8_7,v_g8_7) = 1.0_wp
    GAM1(eq_g8_7,v_g8_8) = 1.0_wp

    GAM0(eq_g8_8,v_g8_8) = 1.0_wp
    PSI(eq_g8_8,e_g8)   = 1.0_wp

    !! zet !!
    GAM0(eq_zet,v_zet)    = 1.0_wp
    GAM1(eq_zet,v_zet)    = rhozet;
    GAM1(eq_zet,v_zet8_1) = 1.0_wp
    PSI(eq_zet,e_zet0) = 1.0_wp

    GAM0(eq_zet8_1,v_zet8_1) = 1.0_wp
    GAM1(eq_zet8_1,v_zet8_2) = 1.0_wp

    GAM0(eq_zet8_2,v_zet8_2) = 1.0_wp
    GAM1(eq_zet8_2,v_zet8_3) = 1.0_wp

    GAM0(eq_zet8_3,v_zet8_3) = 1.0_wp
    GAM1(eq_zet8_3,v_zet8_4) = 1.0_wp

    GAM0(eq_zet8_4,v_zet8_4) = 1.0_wp
    GAM1(eq_zet8_4,v_zet8_5) = 1.0_wp
    PSI(eq_zet8_4,e_zet4)     = 1.0_wp

    GAM0(eq_zet8_5,v_zet8_5) = 1.0_wp
    GAM1(eq_zet8_5,v_zet8_6) = 1.0_wp

    GAM0(eq_zet8_6,v_zet8_6) = 1.0_wp
    GAM1(eq_zet8_6,v_zet8_7) = 1.0_wp

    GAM0(eq_zet8_7,v_zet8_7) = 1.0_wp
    GAM1(eq_zet8_7,v_zet8_8) = 1.0_wp

    GAM0(eq_zet8_8,v_zet8_8) = 1.0_wp
    PSI(eq_zet8_8,e_zet8)   = 1.0_wp


    !!      7. Euler Equation                                !!
    GAM0(eq_Elam, v_lam)  = 1.0_wp
    GAM1(eq_Elam, v_Elam) = 1.0_wp
    PPI(eq_Elam, sh_Elam) = 1.0_wp

    GAM0(eq_Emuy, v_muy)  = 1.0_wp
    GAM1(eq_Emuy, v_Emuy) = 1.0_wp
    PPI(eq_Emuy, sh_Emuy) = 1.0_wp

    GAM0(eq_Emuk, v_muk)  = 1.0_wp
    GAM1(eq_Emuk, v_Emuk) = 1.0_wp
    PPI(eq_Emuk, sh_Emuk) = 1.0_wp

    GAM0(eq_Ei, v_i)  = 1.0_wp
    GAM1(eq_Ei, v_Ei) = 1.0_wp
    PPI(eq_Ei, sh_Ei) = 1.0_wp

    GAM0(eq_Eu, v_u)  = 1.0_wp
    GAM1(eq_Eu, v_Eu) = 1.0_wp
    PPI(eq_Eu, sh_Eu) = 1.0_wp

    GAM0(eq_Ez, v_z)  = 1.0_wp
    GAM1(eq_Ez, v_Ez) = 1.0_wp
    PPI(eq_Ez, sh_Ez) = 1.0_wp

    GAM0(eq_Eh, v_h)  = 1.0_wp
    GAM1(eq_Eh, v_Eh) = 1.0_wp
    PPI(eq_Eh, sh_Eh) = 1.0_wp

    GAM0(eq_Emua, v_mua)  = 1.0_wp
    GAM1(eq_Emua, v_Emua) = 1.0_wp
    PPI(eq_Emua, sh_Emua) = 1.0_wp

    GAM0(eq_Eh, v_h)  = 1.0_wp
    GAM1(eq_Eh, v_Eh) = 1.0_wp
    PPI(eq_Eh, sh_Eh) = 1.0_wp

    GAM0(eq_Eq, v_q)  = 1.0_wp
    GAM1(eq_Eq, v_Eq) = 1.0_wp
    PPI(eq_Eq, sh_Eq) = 1.0_wp

    GAM0(eq_Ep, v_p)  = 1.0_wp
    GAM1(eq_Ep, v_Ep) = 1.0_wp
    PPI(eq_Ep, sh_Ep) = 1.0_wp

    GAM0(eq_Es, v_s)  = 1.0_wp
    GAM1(eq_Es, v_Es) = 1.0_wp
    PPI(eq_Es, sh_Es) = 1.0_wp

    GAM0(eq_Ezet, v_zet)  = 1.0_wp
    GAM1(eq_Ezet, v_Ezet) = 1.0_wp
    PPI(eq_Ezet, sh_Ezet) = 1.0_wp

    GAM0(eq_Ec, v_c)  = 1.0_wp
    GAM1(eq_Ec, v_Ec) = 1.0_wp
    PPI(eq_Ec, sh_Ec) = 1.0_wp

    GAM0(eq_Ev, v_v)  = 1.0_wp
    GAM1(eq_Ev, v_Ev) = 1.0_wp
    PPI(eq_Ev, sh_Ev) = 1.0_wp


    !!      42-46. Lags                                      !!

    GAM0(eq_ylag, v_ylag) = 1.0_wp
    GAM1(eq_ylag, v_y)    = 1.0_wp

    GAM0(eq_clag, v_clag) = 1.0_wp
    GAM1(eq_clag, v_c)    = 1.0_wp

    GAM0(eq_ilag, v_ilag) = 1.0_wp
    GAM1(eq_ilag, v_i)    = 1.0_wp

    GAM0(eq_glag, v_glag) = 1.0_wp
    GAM1(eq_glag, v_g)    = 1.0_wp

    GAM0(eq_xglag, v_xglag) = 1.0_wp
    GAM1(eq_xglag, v_xg)    = 1.0_wp

    GAM0(eq_hlag, v_hlag) = 1.0_wp
    GAM1(eq_hlag, v_h)    = 1.0_wp

    GAM0(eq_zlag, v_zlag) = 1.0_wp
    GAM1(eq_zlag, v_z)    = 1.0_wp 


    !!      QZ(generalized Schur) decomposition by GENSYS
    call do_gensys(TT, CC, RR, fmat, fwt, ywt, gev, eu, loose, & 
         GAM0, GAM1, C, PSI, PPI, DIV)

    info = eu(1)

    do i = 24,44
       QQ(i-23,i-23) = (para(i)/100.0_wp)**2
    end do

    eq_ygr = 1
    eq_cgr = 2
    eq_igr = 3
    eq_hgr = 4
    eq_ggr = 5
    eq_agr = 6
    eq_tfp = 7

    if (datafile == '/mq/home/m1eph00/code/fortran/models/news/sgu_actual.txt') then 
       DD = 0.0_wp
    else
       DD(eq_ygr) = 100.0_wp*log(muyss);
       DD(eq_cgr) = 100.0_wp*log(muyss);
       DD(eq_igr) = 100.0_wp*log(muyss);
       DD(eq_ggr) = 100.0_wp*log(muyss);
       DD(eq_agr) = 100.0_wp*log(muass);
       DD(eq_tfp) = 100.0_wp*(1.0_wp-alpk)*log(muxss);
    endif

    ZZ(eq_ygr, v_y)    =  100.0_wp
    ZZ(eq_ygr, v_ylag) = -100.0_wp
    ZZ(eq_ygr, v_muy)  =  100.0_wp

    ZZ(eq_cgr, v_c)    =  100.0_wp
    ZZ(eq_cgr, v_clag) = -100.0_wp
    ZZ(eq_cgr, v_muy)  =  100.0_wp

    ZZ(eq_igr, v_i)    =  100.0_wp
    ZZ(eq_igr, v_ilag) = -100.0_wp
    ZZ(eq_igr, v_muy)  =  100.0_wp

    ZZ(eq_hgr, v_h)    =  100.0_wp
    ZZ(eq_hgr, v_hlag) = -100.0_wp

    ZZ(eq_ggr, v_g)    =  100.0_wp
    ZZ(eq_ggr, v_glag) = -100.0_wp
    ZZ(eq_ggr, v_muy)  =  100.0_wp
    ZZ(eq_ggr, v_xg)   =  100.0_wp
    ZZ(eq_ggr, v_xglag)= -100.0_wp

    ZZ(eq_agr, v_mua)  =  100.0_wp;

    ZZ(eq_tfp, v_z)    =  100.0_wp;
    ZZ(eq_tfp, v_zlag) = -100.0_wp;
    ZZ(eq_tfp, v_mux)  =  100.0_wp*(1-alpk);

    HH(eq_ygr,eq_ygr) = para(45)**2


  end subroutine sysmat

  function pmsv result(para)

    real(wp) :: para(npara)
    para = (/4.74_wp, 0.01_wp, 9.11_wp, 0.34_wp, 0.91_wp, 0.72_wp, 0.92_wp, 0.48_wp, 0.96_wp, 0.38_wp, 0.98_wp, 0.17_wp, 0.47_wp, 0.225_wp, 0.675_wp, 0.025_wp, 0.99_wp, 0.20_wp, 15.00_wp, 0.9957_wp, 1.00324407709013_wp, 1.0_wp, 0.2_wp, 0.21_wp, 0.16_wp, 0.16_wp, 0.38_wp, 0.08_wp, 0.10_wp, 11.7_wp, 1.93_wp, 5.50_wp, 0.65_wp, 0.11_wp, 0.09_wp, 0.50_wp, 4.79_wp, 0.51_wp, 0.62_wp, 0.57_wp, 0.37_wp, 4.03_wp, 1.89_wp, 2.21_wp, 0.29_wp/) 

  end function pmsv

  function pmsv2 result(para)

    real(wp) :: para(npara)

    para = (/4.174153108179624_wp,   0.000491175153370_wp,  10.059020873004441_wp,   0.384933604049384_wp,   0.892132673793775_wp,   0.633722871348245_wp,   0.935733156703332_wp,   0.465684934715676_wp,   0.993942924939589_wp,   0.878258728874490_wp,   0.946289006622183_wp,   0.188762144410180_wp,   0.477564618045764_wp,   0.225000000000006_wp,   0.675000000000052_wp,   0.025000000000002_wp,   0.989999999999876_wp,   0.200000000000018_wp,   0.149999999999986_wp,   0.995700000000041_wp,   1.003244077090130_wp,   1.000000000000000_wp,   0.200000000000018_wp,   0.238236041859254_wp,   0.123372020676148_wp,   0.143562877052044_wp,   0.318853492198752_wp,   0.131789178634623_wp,   0.121404571661209_wp,  11.998454134767714_wp,   1.051206122817772_wp,   8.146020923652301_wp,   0.668512833017236_wp,   0.118185273249384_wp,   0.118853198018222_wp,   0.514094839081553_wp,  30.936506930854812_wp,   0.486274043359243_wp,   0.398800719775258_wp,   0.957941755284214_wp,   0.122939912694497_wp,   4.333458887110877_wp,   1.788625572424062_wp,   1.772780859891949_wp,   0.299530988347568_wp/)

  end function pmsv2

  function pmsv3 result(para)
    real(wp) :: para(npara)
    para = (/   1.832677758867413_wp,   0.01_wp,   5.770073223482556_wp,   0.387770813741660_wp,   0.835952291680088_wp,   0.708412760382095_wp,   0.960039645531665_wp,   0.460557914769223_wp,   0.988467275400428_wp,   0.817307175399060_wp,   0.980603851735698_wp,   0.180946133618861_wp,   0.990113496883006_wp,   0.225000000000015_wp,   0.675000000000023_wp,   0.025000000000002_wp,   0.989999999999896_wp,   0.200000000000012_wp,   0.149999999999988_wp,   0.995699999999964_wp,   1.003244077090130_wp,   1.000000000000000_wp,   0.200000000000012_wp,   0.267873904205411_wp,   0.127217919971162_wp,   0.116366973755875_wp,   0.343619399463481_wp,   0.102343939033643_wp,   0.122941208773730_wp,   4.389654231543370_wp,   2.049310258759379_wp,   5.220822600887112_wp,   0.662849651358394_wp,   0.128188943421649_wp,   0.113656789857961_wp,   8.441305511007949_wp,   0.484828574637536_wp,   0.076120659249869_wp,   0.309259903852968_wp,   0.939050310491994_wp,   0.371203755019711_wp,   2.146114601945410_wp,   1.264110876533644_wp,   4.435037732926476_wp,   0.299379132719673_wp/)
  end function pmsv3

  function pmsv4 result(para)
    real(wp) :: para(npara)
    para = (/4.474_wp, 0.001_wp, 9.11_wp, 0.34_wp, 0.91_wp, 0.72_wp, 0.92_wp, 0.48_wp, 0.96_wp, 0.38_wp, 0.92_wp, 0.17_wp, 0.47_wp, 0.225000000000000_wp,   0.675000000000000_wp,   0.025000000000000_wp,   0.989999999999896_wp,   0.200000000000012_wp,   0.149999999999988_wp,   0.995699999999964_wp,   1.003244077090130_wp,   1.000000000000000_wp,   0.200000000000012_wp,0.21_wp, 0.16_wp, 0.16_wp, 0.38_wp, 0.08_wp, 0.10_wp, 11.72_wp, 1.93_wp, 5.50_wp, 0.65_wp, 0.11_wp, 0.09_wp, 0.50_wp, 4.79_wp, 0.51_wp, 0.62_wp, 0.57_wp, 0.37_wp, 4.03_wp, 1.89_wp, 2.21_wp, 0.30_wp/);
  end function pmsv4

  function sgu_pmsv result(para)
    real(wp) :: para(npara)
!    para = (/ 5.391074768466587_wp, 0.002185459183159_wp, 25.070737126434985_wp, 0.437424411405766_wp, 0.928096009137826_wp, 0.732796295529521_wp, 0.947889811865346_wp, 0.479642682043620_wp, 0.952236769574842_wp, 0.771551460749151_wp, 0.972156137756487_wp, 0.099915155559631_wp, 0.208756420031805_wp

    para = (/   4.391074768466587_wp, 0.002185459183159_wp, 25.070737126434985_wp, 0.437424411405766_wp,  0.937470716300834_wp, 0.740198278312648_wp,  0.957464456429642_wp,  0.484487557619818_wp,  0.961855322802871_wp, 0.771551460749151_wp, 0.981975896723724_wp, 0.100924399555183_wp, 0.210865070739197_wp, 0.225_wp, 0.675_wp, 0.025_wp, 0.99_wp, 0.20_wp, 1.1500_wp, 0.9957_wp, 1.00324407709013_wp, 1.0_wp, 0.2_wp, 0.162050841738623_wp, 0.197647876901814_wp, 0.187612606612498_wp, 0.451364773252677_wp, 0.116807952048844_wp, 0.122579250799784_wp, 34.813163825347871_wp, 11.986095767601309_wp, 14.908125488732368_wp, 0.620700579189226_wp, 0.112322391783455_wp, 0.113336251904142_wp, 1.511087229244606_wp, 3.932906965349178_wp, 3.198395880370730_wp, 0.531014250285291_wp,  0.688557661528428_wp, 0.434035500465772_wp,  2.827023982975568_wp,  2.760750599253194_wp,  5.338103984144542_wp,  0.299530635386202_wp/)
  end function sgu_pmsv

  function pmsv5 result(para)

    real(wp) :: para(npara)

    para = (/4.5655_wp, 0.4384_wp, 9.1227_wp, 0.5057_wp, 0.9188_wp, 0.7223_wp, 0.9221_wp, 0.4768_wp, 0.9811_wp, 0.8830_wp, 0.9857_wp, 0.2297_wp, 0.6821_wp, 0.2250_wp, 0.6750_wp, 0.0250_wp, 0.9900_wp, 0.2000_wp, 1.1500_wp, 0.9957_wp, 1.0032_wp, 1.0000_wp, 0.2000_wp, 0.0464_wp, 0.1175_wp, 0.3338_wp, 0.4138_wp, 0.0137_wp, 0.1452_wp, 9.4554_wp, 3.1318_wp, 1.2263_wp, 0.4419_wp, 0.4797_wp, 0.2205_wp, 6.4355_wp, 1.6873_wp, 0.2131_wp, 0.0593_wp, 1.0773_wp, 0.4919_wp, 3.0886_wp, 4.1711_wp, 0.9208_wp, 0.2992_wp/)

  end function pmsv5

end module news

  
